-- Master GF Driver
-- Looks after power on/off checking, and runs other Lua's depending on connected units

local models = {} -- for counts of devices of each type
brightness = 10 --default brightness for all units

--##########################################################
-- initial power on/off checking
powerset = 0 -- 1 Batt, 2 Batt>16v, 4 Avionics, 8 Not failed

if ipc.readUD(0x281C) ~= 0 then
    powerset = 1 -- Battery is on to start
end
if ipc.readDBL(0x2834) > 16.0 then
    powerset = logic.Or(powerset, 2) -- Battery is > 16v
end
if ipc.readDBL(0x2E80) ~= 0 then
    powerset = logic.Or(powerset, 4) -- Avionics on
end
if ipc.readUB(0x0B6A) == 0 then
    powerset = logic.Or(powerset, 8) -- Electrics not failed
end

--##########################################################
-- Power on or off
local function setallbright(b)
		m = 0
		while m < 32 do
		    if models[m] ~= 0 then
		        u = 0
		        while u < models[m] do
				    		if m == GFLGT2 then
		 		        		-- to counter problem in LGT2
										-- brightness value is reversed!    	
								    gfd.SetBright(m, u, 15-b)
								else
		                gfd.SetBright(m, u, b)
		            end
		            u = u + 1
		        end
		    end
		    m = m + 1
	  end
end

local function checkpower()
    if powerset == 15 then
    		setallbright(brightness)
    else
    		setallbright(0)
    end
end

-- ########################################################
-- Detecting changes by events on offsets
local function keeptrack(value, n)
   if value == 0 then
      powerset = logic.And(powerset, 15 - n)
   else
      powerset = logic.Or(powerset, n)
   end
   checkpower()
end

-- Battery switch
function batt(offset, value)
		keeptrack(value, 1)
end

-- Battery voltage
function battvolt(offset, value)
    keeptrack(value, 2)
end
      
-- Avionics switch
function avionics(offset, value)
    keeptrack(value, 4)
end

-- Electrical failure
function elecfail(offset, value)
    keeptrack(1 - value, 8)
end

-- ########################################################
-- Main program (executed once only):
gfd.BlankAll()

-- Determine numbers of each device
m = 0
while m < 32 do
	  models[m] = gfd.GetNumDevices(m)
	  if models[m] ~= 0 then
	  		-- Run other Luas here, one for each type connected
	  		luaname = string.format("Lua %s", gfd.GetName(m))
	  		ipc.macro(luaname) 		
		end
	  m = m+1
end

checkpower()

event.offset(0x281C, "UD", "batt")
event.offset(0x2834, "DBL", "battvolt")
event.offset(0x0B6A, "UB", "elecfail")
event.offset(0x2E80, "UD", "avionics")

-- ########################################################
