-- Demonstration of com library HID joystick reading capabilities
-- You can run this with FSUIPC 4.704, 3.992 or WideClient 6.88,
-- or any later versions

-- Set the Vendor and Product names to suit your device.
-- You can have a different Lua running for each device, but be sure
-- that the aes have different names - or else change the offsets
-- used when writing changes to fSUIPC

-- Get the correct names (substrings will do) from FSUIPC or Wideclient 
-- logging. Or use "hidscanner.exe".
-- If the names cannot identify uniquely, use the hexadecimal VID and PID values.

Vendor = 0x16C0
Product = 0x05B5
Device = 0  -- Multiple devices of the same name need increasing Device numbers.

-- Logging on or off (to see when numbers you are getting)
Logging = false

-- Polling rate in number of polls per second
Pollrate = 25

--------------------------------------------------------
-- First, we need to get the device
Report = 0  -- I *think* all joystick types use Inmput Report 0

dev, rd, wrf, wr, init = com.openhid(Vendor, Product, Device, Report)

if dev == 0 then
   ipc.log("Could not open HID")
   ipc.exit()
end

-- These are the tables of values we need for each axis
maxaxes = 12
axes = { "X", "Y", "Z", "R", "U", "V", "Rudder",
			"Slider", "Throttle", "Wheel", "Dial", "Hat" }
axescounts = {}
axesmax = {}

-- We'll support up to 4 of each type (out of the 16 supported in the com library)
-- But this is only for logging / demo purposes. We only send the first to FSUIPC
prevvals = {}
prevvals2 = {}
prevvals3 = {}
prevvals4 = {}
prevvals5 = {}
prevvals6 = {}
prevvals7 = {}
prevvals8 = {}
prevvals9 = {}
prevvals10 = {}
prevvals11 = {}
prevvals12 = {}
prevvals13 = {}
prevvals14 = {}
prevvals15 = {}
prevvals16 = {}
buttons = {}

PrevData = ""

-- Up to 256 buttons = 8 x 32 bit words
prevbuttons = { 0, 0, 0, 0, 0, 0, 0, 0 }

-- Initialise: Find out how many of each axis we have, and log the details
i = 1
while i <= maxaxes do
	 prevvals[i] = 0
   prevvals2[i] = 0
   prevvals3[i] = 0
   prevvals4[i] = 0
   axescounts[i], axesmax[i] = com.GetHidCount(dev, axes[i])
   if axescounts[i] ~= 0 then
      ipc.log("Device has " .. axescounts[i] .. " x " .. axes[i] .. " axis, Max= " .. axesmax[i])
   end
   i = i+1
end

-- Okay: the Polling routine is caled on a time event, set at the end
------------------------------------------------------------------------
function Poll(time)
  -- We use "readlast" so the values we use are the most up-to-date
  CurrentData, n, discards = com.readlast(dev, rd)
    
  -- Uncomment this part to log the numbers of discards
  --if Logging and (discards ~= 0) then
  --   ipc.log("Discarded " .. discards)
  --end
  
  -- Uncomment this part to log real data when it changes
  if n ~= 0 and CurrentData ~= PrevData then
  	PrevData = CurrentData
    logstr = "Data Read="
    i = 1
    while string.byte(CurrentData,i) do
      logstr = logstr .. string.format(" %02X", string.byte(CurrentData,i))
      i = i+1
    end
    ipc.log(logstr)
  end
  
  -- Extract values we need
  if n ~= 0 then
  	  -- analogue (axis) values first:
  		i = 1
  		
  		while i <= maxaxes do
  			 -- check for up to 16 of each of the 12 axis types
  			 if axescounts[i] ~= 0 then
  		      newval, newval2, newval3, newval4,
  		      newval5, newval6, newval7, newval8,
  		      newval9, newval10, newval11, newval12,
  		      newval13, newval14, newval15, newval16
  		       = com.GetHidValue(dev, axes[i], CurrentData)
  		  		if newval ~= prevvals[i] then
  		  			 val = newval
  		  			 
  		  			 --if max value is > 32767 or < 127 need to pre-scale to a more reasonable level
  		  			 if (axesmax[i] < 127) or (axesmax[i] > 32767) then
  		  			    val = (val * 1024) / axesmax[i]
  		  			 end 
  		  			    
  		  		 	 -- Send changes on first such axis to FSUIPC as an assignable axis on joysticks 16-18
  		  		 	 -- i.e. WORD offsets 36A8 onwards
  		  			 ipc.writeUW(0x3ba8 + ((i-1) * 2), val)
  		  	  end
  		  		
  		  		-- optional logging
  			    if Logging and ((newval ~= prevvals[i]) or (newval2 ~= prevvals2[i]) or
  		      			(newval3 ~= prevvals3[i]) or (newval4 ~= prevvals4[i]) or
  		      			(newval5 ~= prevvals5[i]) or (newval6 ~= prevvals6[i]) or
  		      			(newval7 ~= prevvals7[i]) or (newval8 ~= prevvals8[i]) or
  		      			(newval9 ~= prevvals9[i]) or (newval10 ~= prevvals10[i]) or
  		      			(newval11 ~= prevvals11[i]) or (newval12 ~= prevvals12[i]) or
  		      			(newval13 ~= prevvals13[i]) or (newval14 ~= prevvals14[i]) or
  		      			(newval15 ~= prevvals15[i]) or (newval16 ~= prevvals16[i])
  		      			)  then
  					   ipc.log("1-4  =" .. axes[i] .. " = " .. newval .. " " .. newval2 .. " " .. newval3 .. " " .. newval4)
  					   ipc.log("5-8  =" .. axes[i] .. " = " .. newval5 .. " " .. newval6 .. " " .. newval7 .. " " .. newval8)
  					   ipc.log("9-12 =" .. axes[i] .. " = " .. newval9 .. " " .. newval10 .. " " .. newval11 .. " " .. newval12)
  					   ipc.log("12-16=" .. axes[i] .. " = " .. newval13 .. " " .. newval14 .. " " .. newval15 .. " " .. newval16)
  					end
  					prevvals[i] = newval
  					prevvals2[i] = newval2
  					prevvals3[i] = newval3
  					prevvals4[i] = newval4
  					prevvals5[i] = newval5
  					prevvals6[i] = newval6
  					prevvals7[i] = newval7
  					prevvals8[i] = newval8
  					prevvals9[i] = newval9
  					prevvals10[i] = newval10
  					prevvals11[i] = newval11
  					prevvals12[i] = newval12
  					prevvals13[i] = newval13
  					prevvals14[i] = newval14
  					prevvals15[i] = newval15
  					prevvals16[i] = newval16
   			 end
  	     i = i + 1
  	  end
  	  
  	  -- Now handle the buttons : up to 256 of them!
  	  buttons[1], buttons[2], buttons[3], buttons[4],
  	  buttons[5], buttons[6], buttons[7], buttons[8] = 
  	  	com.GetHidButtons(dev, CurrentData)
  	  	
  	  -- check for changes   
    	fbutton = false
    	i = 1
    	while i <= 8 do
    	    if buttons[i] ~= prevbuttons[i] then
    	        fbutton = true -- flag change for logging
    	        prevbuttons[i] = buttons[i]
    	        -- Send to FSUIPC as a set of 32 virtual buttons
    	        -- i.e. DWORD offsets 3340 onwards
    	        ipc.writeUD(0x3340 + ((i-1) * 4), buttons[i])
    	    end
    	    i = i + 1
    	end
    	if Logging and fbutton then
    	    -- log in hexadecimal
    	    ipc.log(string.format("Buttons= %X %X %X %X %X %X %X %X",
    					buttons[1], buttons[2], buttons[3], buttons[4],
  	  				buttons[5], buttons[6], buttons[7], buttons[8]))
  	  end 
  end
end
------------------------------------------------------------------------

if init then
   -- Deal with initial values, if supplied (some joysticks don't)
   ipc.log("init seen!")
   Poll(0)
end

if Pollrate == 0 then
   -- Ouch. Mustn't divide by zero!
   Pollrate = 25 
end

event.timer(1000/Pollrate, "Poll")  -- poll values 'Pollrate' times per second
