-- "Slave Server" example LUA plug-in, by Pete Dowson, October 2009
-- To make the Server FS act as a slave, following the Client FS acting as Master

-- The SOCKET module is built into FSUIPC, but is not active until "required"
socket = require("socket");

-- Set the host name to the name of the PC running this Server
host = "<put the name of your slave PC here>";

-- The port must match the port selected in the client and not clash with others.
port = "8384";


local function processdata(command)

	-- decode string, just numbers in order with non-number separators
	w = string.gmatch(command, "[+-]?[0-9%.?]+")
	lat = w()
	lon = w()
	alt = w()
	pitch = w()
	bank = w()
	hdgT = w()
	
	-- This simply sets the aircraft position and attitude
	-- according to the parameters received

	-- we need to convert to FS units
	hdgT = (((hdgT * 65536) / 360)) * 65536
	lat = (lat / 90) * (10001750 * 65536 * 65536)
	lon = (lon / 360) *  (65536 * 65536 * 65536 * 65536)
	alt = (alt / 3.28084) * (65536 * 65536)
	pitch = (pitch / 360) * (65536 * 65536)
	bank = (bank / 360) * (65536 * 65536)

	-- now write it all in one structure write
	ipc.writeStruct(0x0560,"3DD", lat, lon, alt, "2SD", pitch, bank, "1UD", hdgT)

end

server = assert(socket.bind(host, port));
ack = "\n";
while 1 do
    print("server: waiting for client connection...");
    control = server:accept();
    if control ~= nil then
    		print("server: client connected!");
    		while 1 do 
        		command = control:receive();
        		if command == nil then
            		print("server: client disconnected");
            		ipc.control(65794) -- Pause FS
             		break
        		end
        		assert(control:send(ack));       		
        		processdata(command)
    		end
    end
end
