-- VRInsight example: displays SPD or MACH, by Pete Dowson, February 2010
-- Needs these filters:
   -- WrFilter.1=SPD?
   -- WrFilter.2=SPDOF
   -- WrFilter.3=SPDON
   -- RdFilter.1=SPD?+

if VRImodel == nil then
   -- Set known ports if testing with Lua not loaded automatically
   VRIdriver = "COM2"
   VRIdevice = "COM5"
end

speed = 115200 -- is this the same for all VRI devices?
handshake = 0  -- No handshake
minsize = 8
maxsize = 8    -- VRI seems to use fixed length blocks of 8 bytes

machmode = 0
spdmode = 0

dev = com.open(VRIdevice, speed, handshake)

if dev == 0 then
   ipc.log("Could not open VRIdevice port")
   ipc.exit()
end

local function setmode()
   if machmode or spdmode then
	 		str = "SPDON"
	 else
	    str = "SPDOF"
	 end		
   com.write(dev, str, 8)
   ipc.log("Setmode" .. str)
end

function setspeed(off,val)
	 if not machmode then
	    str = string.format("SPD%03d", val)
	    com.write(dev, str, 8)
	    ipc.log("Setspeed=" .. str)
	 end
end

function setmach(off,val)
	 if machmode then
	 	  val = (val / 655.36) + 0.5 -- rounded
	    str = string.format("SPD%03d", val)
	    com.write(dev, str, 8)
	    ipc.log("Setmach=" .. str)
	 end
end

function setmchmode(off, val)
   machmode = val ~= 0
   if machmode then
      setmach(0x7e8, ipc.readUD(0x7e8))
	 end
	 setmode()

	 if machmode then
	    str = "true"
	 else
	    str = "false"
	 end
	 ipc.log("machmode=" .. str)
end

function setspdmode(off, val)
   spdmode = val ~= 0
   if spdmode then
      setspeed(0x7e2, ipc.readUW(0x7e2))
	 end
	 setmode()

	 if spdmode then
	    str = "true"
	 else
	    str = "false"
	 end
	 ipc.log("spdmode=" .. str)
end

function setmachtofs(handle, str)
	-- need to check only for SPDnnn+/- and send speed or mach to FS
	speed = tonumber(string.match(str, "SPD(%d%d%d)"))
	if speed ~= nil then
			if machmode then
					speed = speed * 655.36
					ipc.writeUD(0x7e8, speed)
			else
					ipc.writeUW(0x7e2, speed)
			end
	end
end

setmchmode(0x7e4, ipc.readUD(0x7e4))
setspdmode(0x7dc, ipc.readUD(0x7dc))

event.offset(0x7e2, "UW", "setspeed")
event.offset(0x7e8, "UD", "setmach")
event.offset(0x7e4, "UD", "setmchmode")
event.offset(0x7dc, "UD", "setspdmode")
event.VRIread(dev, "setmachtofs")
