-- Assorted values
delayperchange = 500
displaymax = 20
local models = {} -- for counts of devices of each type

-- clear down the display
ipc.lineDisplay("", 0)

-- function to test all display types
local function testdisplay(fn, param3, param4)
    m = 0
    while m < 32 do
        u = 0
        if models[m] ~= 0 then
            while u < models[m] do
               fn(m, u, param3, param4)
               u = u + 1
				       ipc.sleep(5) --Added delay, to try to fix missed displays on multiple units
            end
        end
        m = m + 1
    end
end

-- function to format and log results of input events
local function MyLineFormat(name, fn)
		line = string.format(
	        "%s[8]=[%d,%d,%d,%d,%d,%d,%d,%d]",
	        		name,	fn(0), fn(1), fn(2), fn(3),
	                  fn(4), fn(5), fn(6), fn(7))
		ipc.lineDisplay(line, -displaymax)
	  ipc.log(line)
end

-- main input event processing
function checkallinputs(model, unit)
		-- retrieve al the values associated with this mode.unit
    gfd.GetValues(model, unit)
    
    -- log & display the model.unit name and the buttons (in hex)
    ipc.lineDisplay("", -displaymax)
    btns = string.format(
	        "%s.%d: Btns[32]=%08X",
	        		gfd.GetName(model), unit, gfd.Buttons())
	  ipc.lineDisplay(btns, -displaymax)      		
	  ipc.log(btns)
	  
	  
	  -- Log and display all the other values -- dials, selectors, levers
	  MyLineFormat("Dials", gfd.Dial)
	  MyLineFormat("Selectors", gfd.Selector)
	  MyLineFormat("Levers", gfd.Lever)
end

-- Main program (executed once only):

-- Determine numbers of each device
m = 0
while m < 32 do
	  models[m] = gfd.GetNumDevices(m)
	  if models[m] ~= 0 then
		    ipc.log(string.format("No. of %s = %d", gfd.GetName(m), models[m]))
		end
	  m = m+1
	  
end

-- Display tests (lasts many seconds depending on "delayperchange")

-- LEDs all on
ipc.lineDisplay("All displays:", -displaymax)
ipc.lineDisplay("  brightness adjustment ...", -displaymax)
ipc.lineDisplay("", -displaymax)
testdisplay(gfd.SetLights, -1, 0)

-- Displays all 8's
id = 0
while id < 8 do
    testdisplay(gfd.SetDisplay, id, "888888")
    id = id + 1
end

-- Brightness test (dim to bright, 16 steps)
b = 0
while b < 16 do
    ipc.sleep(delayperchange)
    ipc.lineDisplay("Brightess " .. b, -displaymax)
    testdisplay(gfd.SetBright, b)
    b = b + 1
end

-- Alternate LEDs or 012345 / 543210
ipc.lineDisplay("", -displaymax)
ipc.lineDisplay("All displays:", -displaymax)
ipc.lineDisplay("  value setting ...", -displaymax)
ipc.lineDisplay("", -displaymax)
i = 0
lpat1 = 0x55555555
lpat2 = 0xaaaaaaaa
dpat1 = "123456"
dpat2 = "654321"

while i < 8 do
    id = 0
    ipc.lineDisplay(string.format("Display %s", dpat1), -displaymax)
    ipc.lineDisplay(string.format("Lights %08X", lpat1), -displaymax)
    while id < 8 do
        testdisplay(gfd.SetDisplay, id, dpat1)
        id = id + 1
        ipc.sleep(5) --Added delay, to try to fix missed displays on multiple units
    end
    testdisplay(gfd.SetLights, lpat1, lpat2)
    ipc.sleep(delayperchange)
    
    lpat1, lpat2 = lpat2, lpat1
    dpat1, dpat2 = dpat2, dpat1
    i = i + 1
end

-- okay, blank the displays and end with
-- accepting and logging/displaying inputs

gfd.BlankAll()
  
-- Now inputs, by event
ipc.lineDisplay("", -displaymax)
ipc.lineDisplay("Now Ready for input tests ...", -displaymax)
ipc.lineDisplay("", -displaymax)
event.gfd("checkallinputs")

-----------------------------------------
